;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10; Package: modlisp -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          base.lisp
;;;; Purpose:       Utility functions for modlisp package
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  Dec 2002
;;;;
;;;; $Id: base.lisp,v 1.3 2003/07/05 00:51:04 kevin Exp $
;;;; *************************************************************************

(in-package #:modlisp)


(let ((*listener-socket* nil)
      (*listener-process* nil))

  (defun modlisp-start (&key (port +default-apache-port+)
			     (function 'demo-apache-command-processor))
    (handler-case
	(make-socket-server (next-server-name) function port
			    :format :text :wait nil)
      (error (e)
	(format t "~&Error while trying to start modlisp server~&  ~A" e)
	(decf *listener-count*)
	nil)
      (:no-error (process socket)
	(setq *listener-socket* socket)
	(setq *listener-process* process)
	process)))
  
  (defun modlisp-stop ()
    (when *listener-process*
      (format t "~&; killing modlisp server process ~A~%" *listener-process*)
      (handler-case
	  (progn
	    #+sbcl (sb-thread:destroy-thread *listener-process*)
	    #+cmucl (mp:destroy-process *listener-process*)
	    #+allegro (mp:process-kill *listener-process*)
	    #+allegro (mp:process-allow-schedule)
	    #+lispworks (mp:process-kill *listener-process*))
	(error (e)
	  (format t "~&Error while trying to kill modlisp server~&  ~A" e))
	(:no-error (res)
	  (declare (ignore res))
	  (setq *listener-process* nil))))
    (when *listener-socket*
      (ignore-errors (close *listener-socket*))
      (setq *listener-socket* nil)))
  
 ) ;; closure
    
(defun next-server-name ()
  (format nil "modlisp-socket-server-~d" (incf *listener-count*))) 

(defun next-worker-name ()
  (format nil "modlisp-worker-~d" (incf *worker-count*)))

(let ((*number-server-requests* 0)
      (*number-worker-requests* 0)
      (*close-apache-socket* t))
  
  (defun apache-command-issuer (*apache-socket* processor-fun)
    "generates commands from apache, issues commands to processor-fun"
    (unwind-protect
	 (progn
	   (setq *number-worker-requests* 0)
	   (do ((command (read-apache-command) (read-apache-command)))
	       ((null command) 'done)
	     (funcall processor-fun command)
	     (force-output *apache-socket*)
	     (incf *number-worker-requests*)
	     (incf *number-server-requests*)
	     (when *close-apache-socket*
	       (return))))
      (close *apache-socket*)))

  (defun get-number-worker-requests ()
    *number-worker-requests*)

  (defun get-number-server-requests ()
    *number-server-requests*)
  
  (defun set-close-apache-socket (close?)
    (setq *close-apache-socket* close?))
  
  ) ;; closure

(defun read-apache-command ()
  (ignore-errors
    (let* ((header (read-apache-header))
	  (content-length (cdr (assoc "content-length" header :test #'equal)))
	  (content (when content-length 
		     (make-string
		      (parse-integer content-length :junk-allowed t)))))
     (when content
       (read-sequence content *apache-socket*)
       (push (cons "posted-content" content) header))
     header)))

(defun read-apache-header ()
  (loop for key = (read-line *apache-socket* nil nil)
			while (and key
				   (string-not-equal key "end")
				   (> (length key) 1))
		      for value = (read-line *apache-socket* nil nil)
		      collect (cons key value)))

(defun write-header-line (key value)
  (write-string key *apache-socket*)
  (write-char #\NewLine *apache-socket*)
  (write-string value *apache-socket*)
  (write-char #\NewLine *apache-socket*))

(defun header-value (command key)
  (cdr (assoc key command :test #'string=)))

